@@ -18,12 +18,16 @@
 #ifndef _SMALLUT_H_INCLUDED_
 #define _SMALLUT_H_INCLUDED_
 
+#ifdef __MINGW32__
+#define _POSIX_C_SOURCE 200809L
+#endif
+
+#include <cstdint>
+#include <ctime>
 #include <functional>
 #include <map>
 #include <memory>
-#include <stdint.h>
 #include <string>
-#include <time.h>
 #include <vector>
 
 struct tm;
@@ -44,7 +48,7 @@ void smallut_init_mt();
 #define MAX(A,B) (((A)>(B)) ? (A) : (B))
 #endif
 #ifndef deleteZ
-#define deleteZ(X) {delete X;X = nullptr;}
+#define deleteZ(X) do {delete X;X = nullptr;} while(0)
 #endif
 #ifndef PRETEND_USE
 #define PRETEND_USE(var) ((void)(var))
@@ -66,7 +70,7 @@ struct StringIcmpPred {
     explicit StringIcmpPred(const std::string& s1)
         : m_s1(s1) {
     }
-    bool operator()(const std::string& s2) {
+    bool operator()(const std::string& s2) const {
         return stringicmp(m_s1, s2) == 0;
     }
     const std::string& m_s1;
@@ -82,29 +86,19 @@ extern std::string stringtolower(const std::string& io);
 extern void stringtoupper(std::string& io);
 extern std::string stringtoupper(const std::string& io);
 extern bool beginswith(const std::string& b, const std::string& sml);
+extern bool endswith(const std::string& bg, const std::string& sml);
 
-// Parse date interval specifier into pair of y,m,d dates.  The format
-// for the time interval is based on a subset of iso 8601 with
-// the addition of open intervals, and removal of all time indications.
-// 'P' is the Period indicator, it's followed by a length in
-// years/months/days (or any subset thereof)
-// Dates: YYYY-MM-DD YYYY-MM YYYY
-// Periods: P[nY][nM][nD] where n is an integer value.
-// At least one of YMD must be specified
-// The separator for the interval is /. Interval examples
-// YYYY/ (from YYYY) YYYY-MM-DD/P3Y (3 years after date) etc.
-// This returns a pair of y,m,d dates.
-struct DateInterval {
-    int y1;
-    int m1;
-    int d1;
-    int y2;
-    int m2;
-    int d2;
-};
-extern bool parsedateinterval(const std::string& s, DateInterval *di);
-extern int monthdays(int mon, int year);
-
+#ifdef _WIN32
+// Conversion between utf-8 and wide char file names.
+bool wchartoutf8(const wchar_t *in, std::string& out, int len = 0);
+std::string wchartoutf8(const wchar_t *in, int len = 0);
+bool utf8towchar(const std::string& in, wchar_t *out, int obytescap);
+std::unique_ptr<wchar_t[]> utf8towchar(const std::string& in);
+#define strcasecmp _stricmp
+#define strncasecmp _strnicmp
+#define localtime_r(a,b) localtime_s(b,a)
+#define strtok_r strtok_s
+#endif // _WIN32
 
 /** Note for all templated functions:
  * By default, smallut.cpp has explicit instantiations for common
@@ -296,6 +290,8 @@ extern std::string pc_decode(const std::string&);
 /// start meaning count from the end).
 bool parseHTTPRanges(const std::string& ranges, std::vector<std::pair<int64_t, int64_t>>& oranges);
 
+void millisleep(int millis);
+
 
 } // End namespace MedocUtils
 
