@@ -74,93 +74,131 @@ public:
 /* Worker to send the search messages. */
 class SearchSendJobWorker : public JobWorker {
 public:
-    explicit SearchSendJobWorker(std::string v4, std::string v6)
-        : ReqBufv4(std::move(v4)), ReqBufv6(std::move(v6)) {}
-    std::string ReqBufv4;
-    std::string ReqBufv6;
-    void work() override;
+    explicit SearchSendJobWorker(const NetIF::Interface& _iface, SOCKET& _sockRef,std::string reqBuf)
+        : iface(_iface), sockRef(_sockRef), ReqBuff(std::move(reqBuf)) {}
 
+    const NetIF::Interface& iface;
+    SOCKET& sockRef;
+    std::string ReqBuff;
+    bool socketReady();
 };
 
-void SearchSendJobWorker::work()
-{
-    bool needv4 = !ReqBufv4.empty();
-    bool needv6 = !ReqBufv6.empty();
-    fd_set wrSet;
-    FD_ZERO(&wrSet);
-    int max_fd = -1;
-    if (needv4) {
-        if (gSsdpReqSocket4 != INVALID_SOCKET) {
-            FD_SET(gSsdpReqSocket4, &wrSet);
-#ifdef _WIN32
-            // max_fd is ignored under windows. Just set it to not -1.
-            max_fd = 0;
-#else
-            max_fd = std::max(max_fd, gSsdpReqSocket4);
-#endif
-        }
-    }
-#ifdef UPNP_ENABLE_IPV6
-    if (needv6) {
-        if (gSsdpReqSocket6 != INVALID_SOCKET) {
-            FD_SET(gSsdpReqSocket6, &wrSet);
-#ifdef _WIN32
-            // max_fd is ignored under windows. Just set it to not -1.
-            max_fd = 0;
-#else
-            max_fd = std::max(max_fd, gSsdpReqSocket6);
-#endif
-        }
-    }
-#endif
+class SearchSendJobWorkerV4 : public SearchSendJobWorker {
+public:
+    explicit SearchSendJobWorkerV4(const NetIF::Interface& iface, SOCKET& sockRef,std::string reqBuf)
+        : SearchSendJobWorker(iface, sockRef, std::move(reqBuf)) {}
 
-    if (max_fd == -1) {
-        UpnpPrintf(UPNP_ERROR, SSDP, __FILE__, __LINE__,
-                   "SSDP_LIB: neither ipv4 nor ipv6 are active !\n");
-        return;
-    }
-    int ret = select(max_fd + 1, nullptr, &wrSet, nullptr, nullptr);
-    if (ret == -1) {
-        char errorBuffer[ERROR_BUFFER_LEN];
-        posix_strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
-        UpnpPrintf(UPNP_ERROR, SSDP, __FILE__, __LINE__,
-                   "SSDP_LIB: Error in select(): %s\n", errorBuffer);
-        UpnpCloseSocket(gSsdpReqSocket4);
-#ifdef UPNP_ENABLE_IPV6
-        UpnpCloseSocket(gSsdpReqSocket6);
-#endif
-        return;
-    }
+    void work() override;
+};
 
 #ifdef UPNP_ENABLE_IPV6
-    if (needv6 && gSsdpReqSocket6 != INVALID_SOCKET && FD_ISSET(gSsdpReqSocket6, &wrSet)) {
-        struct sockaddr_storage ssv6 = {};
-        auto destAddr6 = reinterpret_cast<struct sockaddr_in6 *>(&ssv6);
-        destAddr6->sin6_family = static_cast<sa_family_t>(AF_INET6);
-        inet_pton(AF_INET6, SSDP_IPV6_LINKLOCAL, &destAddr6->sin6_addr);
-        destAddr6->sin6_port = htons(SSDP_PORT);
-        destAddr6->sin6_scope_id = apiFirstIPV6Index();
-        UpnpPrintf(UPNP_DEBUG, SSDP, __FILE__, __LINE__,
-                   ">>> SSDP SEND M-SEARCH >>>\n%s\n", ReqBufv6.c_str());
-        sendto(gSsdpReqSocket6, ReqBufv6.c_str(), ReqBufv6.size(), 0,
-               reinterpret_cast<struct sockaddr *>(destAddr6), sizeof(struct sockaddr_in6));
-    }
+class SearchSendJobWorkerV6 : public SearchSendJobWorker {
+public:
+    explicit SearchSendJobWorkerV6(const NetIF::Interface& iface, SOCKET& sockRef, std::string reqBuf)
+        : SearchSendJobWorker(iface, sockRef, std::move(reqBuf)) {}
+
+    void work() override;
+};
 #endif /* UPNP_ENABLE_IPV6 */
 
-    if (needv4 && gSsdpReqSocket4 != INVALID_SOCKET && FD_ISSET(gSsdpReqSocket4, &wrSet)) {
-        struct sockaddr_storage ssv4 = {};
-        auto destAddr4 = reinterpret_cast<struct sockaddr_in *>(&ssv4);
-        destAddr4->sin_family = static_cast<sa_family_t>(AF_INET);
-        inet_pton(AF_INET, SSDP_IP, &destAddr4->sin_addr);
-        destAddr4->sin_port = htons(SSDP_PORT);
-        UpnpPrintf(UPNP_DEBUG, SSDP, __FILE__, __LINE__,
-                   ">>> SSDP SEND M-SEARCH >>>\n%s\n", ReqBufv4.c_str());
-        sendto(gSsdpReqSocket4, ReqBufv4.c_str(), ReqBufv4.size(), 0,
-               reinterpret_cast<struct sockaddr *>(destAddr4), sizeof(struct sockaddr_in));
+bool SearchSendJobWorker::socketReady()
+{
+    fd_set wrSet;
+    FD_ZERO(&wrSet);
+
+    if (sockRef == INVALID_SOCKET) {
+        UpnpPrintf(UPNP_ERROR, SSDP, __FILE__, __LINE__,
+                   "SSDP_LIB - %s: socket is invalid !\n", iface.getfriendlyname().c_str());
+        return false;
+    }
+
+    FD_SET(sockRef, &wrSet);
+#ifdef _WIN32
+    // max_fd is ignored under windows. Just set it to not -1.
+    int max_fd = 0;
+#else
+    int max_fd = sockRef;
+#endif
+
+    int ret = select(max_fd + 1, nullptr, &wrSet, nullptr, nullptr);
+    if (ret == -1) {
+        int lastError;
+        std::string errorDesc;
+        NetIF::getLastError(errorDesc, &lastError);
+        UpnpPrintf(UPNP_ERROR, SSDP, __FILE__, __LINE__,
+                   "SSDP_LIB - %s: Error in select():  %d, %s\n", iface.getfriendlyname().c_str(),
+                   lastError, errorDesc.c_str());
+        UpnpCloseSocket(sockRef);
+        return false;
+    }
+
+    if (!FD_ISSET(sockRef, &wrSet)) {
+        UpnpPrintf(UPNP_ERROR, SSDP, __FILE__, __LINE__,
+                   "SSDP_LIB - %s: socket not set after select!\n", iface.getfriendlyname().c_str());
+        return false;
+    }
+    return true;
+}
+
+void SearchSendJobWorkerV4::work()
+{
+    if (!socketReady())
+        return;
+    
+    struct sockaddr_storage ssv4 = {};
+    auto destAddr4 = reinterpret_cast<struct sockaddr_in *>(&ssv4);
+    destAddr4->sin_family = static_cast<sa_family_t>(AF_INET);
+    inet_pton(AF_INET, SSDP_IP, &destAddr4->sin_addr);
+    destAddr4->sin_port = htons(SSDP_PORT);
+    UpnpPrintf(UPNP_ERROR, SSDP, __FILE__, __LINE__,
+               ">>> SSDP SEND M-SEARCH >>>\n%s\n%s\n", iface.getfriendlyname().c_str(),
+               ReqBuff.c_str());
+    auto ret = sendto(sockRef, ReqBuff.c_str(), ReqBuff.size(), 0,
+                      reinterpret_cast<struct sockaddr *>(destAddr4), sizeof(struct sockaddr_in));
+
+    if (ret == -1) {
+        int lastError;
+        std::string errorDesc;
+        NetIF::getLastError(errorDesc, &lastError);
+        UpnpPrintf(UPNP_ERROR, SSDP, __FILE__, __LINE__,
+                   "SSDP_LIB - %s: Error in sendto(): %d, %s\n", iface.getfriendlyname().c_str(),
+                   lastError, errorDesc.c_str());
+        return;
     }
 }
 
-void ssdp_handle_ctrlpt_msg(SSDPPacketParser& parser, const struct sockaddr_storage *dest_addr, void *)
+#ifdef UPNP_ENABLE_IPV6
+void SearchSendJobWorkerV6::work()
+{
+    if (!socketReady())
+        return;
+
+    struct sockaddr_storage ssv6 = {};
+    auto destAddr6 = reinterpret_cast<struct sockaddr_in6*>(&ssv6);
+    destAddr6->sin6_family = static_cast<sa_family_t>(AF_INET6);
+    inet_pton(AF_INET6, SSDP_IPV6_LINKLOCAL, &destAddr6->sin6_addr);
+    destAddr6->sin6_port = htons(SSDP_PORT);
+    destAddr6->sin6_scope_id = iface.getindex();
+    UpnpPrintf(UPNP_DEBUG, SSDP, __FILE__, __LINE__,
+               ">>> SSDP SEND M-SEARCH >>>\n%s\n%s\n", iface.getfriendlyname().c_str(),
+               ReqBuff.c_str());
+    auto ret = sendto(sockRef, ReqBuff.c_str(), ReqBuff.size(), 0,
+                      reinterpret_cast<struct sockaddr*>(destAddr6), sizeof(struct sockaddr_in6));
+
+    if (ret == -1) {
+        int lastError;
+        std::string errorDesc;
+        NetIF::getLastError(errorDesc, &lastError);
+        UpnpPrintf(UPNP_ERROR, SSDP, __FILE__, __LINE__,
+                   "SSDP_LIB - %s: Error in sendto(): %d, %s\n", iface.getfriendlyname().c_str(),
+                   lastError, errorDesc.c_str());
+        return;
+    }
+}
+#endif /* UPNP_ENABLE_IPV6 */
+
+void ssdp_handle_ctrlpt_msg(SSDPPacketParser& parser, const struct sockaddr_storage *dest_addr,
+                            void *)
 {
     int handle;
     struct Handle_Info *ctrlpt_info = nullptr;
@@ -175,16 +213,16 @@ void ssdp_handle_ctrlpt_msg(SSDPPacketParser& parser, const struct sockaddr_stor
     void *ctrlpt_cookie;
     int matched = 0;
 
-    /* Get client info. We are assuming that there can be only one
-       client supported at a time */
-    HandleReadLock();
-    if (GetClientHandleInfo(&handle, &ctrlpt_info) != HND_CLIENT) {
-        HandleUnlock();
-        return;
+    {    
+        HANDLELOCK();
+        /* Get client info. We are assuming that there can be only one
+           client supported at a time */
+        if (GetClientHandleInfo(&handle, &ctrlpt_info) != HND_CLIENT) {
+            return;
+        }
+        ctrlpt_callback = ctrlpt_info->Callback;
+        ctrlpt_cookie = ctrlpt_info->Cookie;
     }
-    ctrlpt_callback = ctrlpt_info->Callback;
-    ctrlpt_cookie = ctrlpt_info->Cookie;
-    HandleUnlock();
 
     param.ErrCode = UPNP_E_SUCCESS;
     /* MAX-AGE, assume error */
@@ -297,50 +335,50 @@ void ssdp_handle_ctrlpt_msg(SSDPPacketParser& parser, const struct sockaddr_stor
             strlen(param.Location) == 0 || !usn_found || !st_found) {
             return;    /* bad reply */
         }
-        /* check each current search */
-        HandleLock();
-        if (GetClientHandleInfo(&handle, &ctrlpt_info) != HND_CLIENT) {
-            HandleUnlock();
-            return;
-        }
-        size_t stlen = strlen(parser.st);
-        for (const auto& searchArg : ctrlpt_info->SsdpSearchList) {
-            /* check for match of ST header and search target */
-            switch (searchArg.requestType) {
-            case SSDP_ALL:
-                matched = 1;
-                break;
-            case SSDP_ROOTDEVICE:
-                matched = (event.RequestType == SSDP_ROOTDEVICE);
-                break;
-            case SSDP_DEVICEUDN:
-                matched = !strncmp(searchArg.searchTarget.c_str(), parser.st, stlen);
-                break;
-            case SSDP_DEVICETYPE:
-            {
-                size_t m = std::min(stlen, searchArg.searchTarget.size());
-                matched = !strncmp(searchArg.searchTarget.c_str(), parser.st, m);
-                break;
+        {
+            /* check each current search */
+            HANDLELOCK();
+            if (GetClientHandleInfo(&handle, &ctrlpt_info) != HND_CLIENT) {
+                return;
             }
-            case SSDP_SERVICE:
-            {
-                size_t m = std::min(stlen, searchArg.searchTarget.size());
-                matched = !strncmp(searchArg.searchTarget.c_str(), parser.st, m);
-                break;
-            }
-            default:
-                matched = 0;
-                break;
-            }
-            if (matched) {
-                /* schedule call back */
-                auto threadData = std::make_unique<ResultData>(param, searchArg.cookie, ctrlpt_callback);
-                auto worker = std::make_unique<SearchResultJobWorker>(std::move(threadData));
-                gRecvThreadPool.addJob(std::move(worker));
+            size_t stlen = strlen(parser.st);
+            for (const auto& searchArg : ctrlpt_info->SsdpSearchList) {
+                /* check for match of ST header and search target */
+                switch (searchArg.requestType) {
+                case SSDP_ALL:
+                    matched = 1;
+                    break;
+                case SSDP_ROOTDEVICE:
+                    matched = (event.RequestType == SSDP_ROOTDEVICE);
+                    break;
+                case SSDP_DEVICEUDN:
+                    matched = !strncmp(searchArg.searchTarget.c_str(), parser.st, stlen);
+                    break;
+                case SSDP_DEVICETYPE:
+                {
+                    size_t m = std::min(stlen, searchArg.searchTarget.size());
+                    matched = !strncmp(searchArg.searchTarget.c_str(), parser.st, m);
+                    break;
+                }
+                case SSDP_SERVICE:
+                {
+                    size_t m = std::min(stlen, searchArg.searchTarget.size());
+                    matched = !strncmp(searchArg.searchTarget.c_str(), parser.st, m);
+                    break;
+                }
+                default:
+                    matched = 0;
+                    break;
+                }
+                if (matched) {
+                    /* schedule call back */
+                    auto threadData = std::make_unique<ResultData>(param, searchArg.cookie,
+                                                                   ctrlpt_callback);
+                    auto worker = std::make_unique<SearchResultJobWorker>(std::move(threadData));
+                    gRecvThreadPool.addJob(std::move(worker));
+                }
             }
         }
-
-        HandleUnlock();
     }
 }
 
@@ -390,36 +428,38 @@ static int CreateClientRequestPacket(
 
 class SearchExpiredJobWorker : public JobWorker {
 public:
+    SearchExpiredJobWorker(int id) : m_id(id) {}
     void work() override;
     int m_id;
 };
 
 void SearchExpiredJobWorker::work()
 {
-    HandleLock();
-
     int handle;
     struct Handle_Info *ctrlpt_info;
-    if (GetClientHandleInfo(&handle, &ctrlpt_info) != HND_CLIENT) {
-        HandleUnlock();
-        return;
-    }
+    Upnp_FunPtr ctrlpt_callback;
+    void *cookie;
+    bool found{false};
+    {
+        HANDLELOCK();
+        if (GetClientHandleInfo(&handle, &ctrlpt_info) != HND_CLIENT) {
+            return;
+        }
 
-    Upnp_FunPtr ctrlpt_callback = ctrlpt_info->Callback;
-    void* cookie = nullptr;
-    int found = 0;
-    int id = m_id;
-    auto it = std::find_if(
-        ctrlpt_info->SsdpSearchList.begin(),
-        ctrlpt_info->SsdpSearchList.end(),
-        [id](const auto& item) { return item.timeoutEventId == id; });
+        ctrlpt_callback = ctrlpt_info->Callback;
+        cookie = nullptr;
+        int id = m_id;
+        auto it = std::find_if(
+            ctrlpt_info->SsdpSearchList.begin(),
+            ctrlpt_info->SsdpSearchList.end(),
+            [id](const auto& item) { return item.timeoutEventId == id; });
 
-    if (it != ctrlpt_info->SsdpSearchList.end()) {
-        cookie = it->cookie;
-        found = 1;
-        ctrlpt_info->SsdpSearchList.erase(it);
+        if (it != ctrlpt_info->SsdpSearchList.end()) {
+            cookie = it->cookie;
+            found = true;
+            ctrlpt_info->SsdpSearchList.erase(it);
+        }
     }
-    HandleUnlock();
 
     if (found)
         ctrlpt_callback(UPNP_DISCOVERY_SEARCH_TIMEOUT, nullptr, cookie);
@@ -463,36 +503,62 @@ int SearchByTarget(int Mx, const char *St, const char *saddress, int port, void
         if (retVal != UPNP_E_SUCCESS)
             return retVal;
     }
-#else
-    if (needv6) {
-        return UPNP_E_INVALID_PARAM;
-    }
 #endif
 
     /* Add the search criteria and callback to the list and schedule a timeout event to remove
        it when the search window closes */
-    HandleLock();
-    int handle;
-    struct Handle_Info *ctrlpt_info;
-    if (GetClientHandleInfo(&handle, &ctrlpt_info) != HND_CLIENT) {
-        HandleUnlock();
-        return UPNP_E_INTERNAL_ERROR;
+    {
+        HANDLELOCK();
+        int handle;
+        struct Handle_Info *ctrlpt_info;
+        if (GetClientHandleInfo(&handle, &ctrlpt_info) != HND_CLIENT) {
+            return UPNP_E_INTERNAL_ERROR;
+        }
+        auto worker = std::make_unique<SearchExpiredJobWorker>(0);
+        int *idp = &(worker->m_id);
+        gTimerThread->schedule(TimerThread::SHORT_TERM, TimerThread::REL_SEC, Mx ? Mx + 1 : 2,
+                               idp, std::move(worker));
+        ctrlpt_info->SsdpSearchList.emplace_back(*idp, St, Cookie, requestType);
     }
-    auto worker = std::make_unique<SearchExpiredJobWorker>();
-    int *idp = &(worker->m_id);
-    gTimerThread->schedule(TimerThread::SHORT_TERM, TimerThread::REL_SEC, Mx ? Mx + 1 : 2,
-                           idp, std::move(worker));
-    ctrlpt_info->SsdpSearchList.emplace_back(*idp, St, Cookie, requestType);
 
-    HandleUnlock();
-
-    // Schedule sending the packets.
+    // Sanity checks
+    if (g_netifs.size() != miniServerGetReqSocks4().size()) {
+        UpnpPrintf(UPNP_ERROR, SSDP, __FILE__, __LINE__,
+                   "g_netifs and SsdpReqSocket4 array sizes differ\n");
+        return UPNP_E_INTERNAL_ERROR;
+    }                
+#ifdef UPNP_ENABLE_IPV6
+    if (g_netifs.size() != miniServerGetReqSocks6().size()) {
+        UpnpPrintf(UPNP_ERROR, SSDP, __FILE__, __LINE__,
+                   "g_netifs and SsdpReqSocket6 array sizes differ\n");
+        return UPNP_E_INTERNAL_ERROR;
+    }                
+#endif // UPNP_ENABLE_IPV6
+    
+    // Schedule sending the search packets.
     std::unique_ptr<SearchSendJobWorker> sworker;
     int delay = 0;
     for (int i = 0; i < NUM_SSDP_COPY; i++) {
-        sworker = std::make_unique<SearchSendJobWorker>(ReqBufv4, ReqBufv6);
-        gTimerThread->schedule(TimerThread::SHORT_TERM, std::chrono::milliseconds(delay),
-                               nullptr, std::move(sworker));
+
+        for (unsigned int ifidx = 0; ifidx < g_netifs.size(); ifidx++) {
+            SOCKET& sockRef{miniServerGetReqSocks4()[ifidx]};
+            if (sockRef == INVALID_SOCKET)
+                continue;
+            sworker = std::make_unique<SearchSendJobWorkerV4>(g_netifs[ifidx], sockRef, ReqBufv4);
+            gTimerThread->schedule(TimerThread::SHORT_TERM, std::chrono::milliseconds(delay),
+                                   nullptr, std::move(sworker));
+        }
+#ifdef UPNP_ENABLE_IPV6
+        for (unsigned int ifidx = 0; ifidx < g_netifs.size(); ifidx++) {
+            SOCKET& sockRef{miniServerGetReqSocks6()[ifidx]};
+            if (sockRef == INVALID_SOCKET)
+                continue;
+            sworker = std::make_unique<SearchSendJobWorkerV6>(g_netifs[ifidx], sockRef, ReqBufv6);
+            gTimerThread->schedule(TimerThread::SHORT_TERM, std::chrono::milliseconds(delay),
+                                   nullptr, std::move(sworker));
+        }
+#endif /* UPNP_ENABLE_IPV6 */
+
         delay += SSDP_PAUSE;
     }
     return UPNP_E_SUCCESS;
