@@ -15,6 +15,12 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  *   02110-1301 USA
  */
+#ifdef BUILDING_RECOLL
+#include "autoconfig.h"
+#else
+#include "config.h"
+#endif
+
 #include "smallut.h"
 
 #include <algorithm>
@@ -33,22 +39,146 @@
 #include <utility>
 #include <vector>
 
-// Older compilers don't support stdc++ regex, but Windows does not have the Linux one. Have a
-// simple class to solve the simple cases.
-#if defined(_WIN32)
+// std::regex makes the program text is almost 100kb bigger than the classical regex.h so we're
+// keeping the latter wherever we can (also, older platforms have no or a buggy std::regex)
+// Windows does not have the classical regex, no choice there.
+// We define a class to solve the simple cases.
+#if __has_include(<regex.h>)
+#include <regex.h>
+#else
 #define USE_STD_REGEX
 #include <regex>
-#define strncasecmp _strnicmp
-#define strcasecmp _stricmp
-#define localtime_r(a,b) localtime_s(b,a)
+#endif
+
+
+#ifdef _WIN32
+
+#define WIN32_LEAN_AND_MEAN
+#define NOGDI
+#include <windows.h>
+#include <stringapiset.h>
+
+#endif // _WIN32
+
+#ifdef MDU_INCLUDE_LOG
+#include MDU_INCLUDE_LOG
 #else
-#include <regex.h>
+#include "log.h"
 #endif
 
 using namespace std::placeholders;
 
 namespace MedocUtils {
 
+#ifdef _WIN32
+
+std::string wchartoutf8(const wchar_t *in, int len)
+{
+    std::string out;
+    wchartoutf8(in, out, len);
+    return out;
+}
+
+bool wchartoutf8(const wchar_t *in, std::string& out, int wlen)
+{
+    out.clear();
+    if (nullptr == in) {
+        return true;
+    }
+    if (wlen == 0) {
+        wlen = static_cast<int>(wcslen(in));
+    }
+    if (wlen == 0) {
+        return true;
+    }
+    int flags = WC_ERR_INVALID_CHARS;
+    int bytes = ::WideCharToMultiByte(CP_UTF8, flags, in, wlen, nullptr, 0, nullptr, nullptr);
+    if (bytes <= 0) {
+#ifdef LOGERR
+        LOGERR("wchartoutf8: conversion error1\n");
+#endif
+        fwprintf(stderr, L"wchartoutf8: conversion error1 for [%s]\n", in);
+        return false;
+    }
+    DirtySmartBuf buffer(bytes+1);
+    bytes = ::WideCharToMultiByte(CP_UTF8, flags, in, wlen, buffer.buf(), bytes, nullptr, nullptr);
+    if (bytes <= 0) {
+#ifdef LOGERR
+        LOGERR("wchartoutf8: CONVERSION ERROR2\n");
+#endif
+        return false;
+    }
+    buffer.buf()[bytes] = 0;
+    out = buffer.buf();
+    //fwprintf(stderr, L"wchartoutf8: in: [%s]\n", in);
+    //fprintf(stderr, "wchartoutf8: out:  [%s]\n", out.c_str());
+    return true;
+}
+
+bool utf8towchar(const std::string& in, wchar_t *out, int obytescap)
+{
+    auto wcharsavail = obytescap / sizeof(wchar_t);
+    if (nullptr == out || wcharsavail < 1) {
+        return false;
+    }
+    out[0] = 0;
+    int isize = static_cast<int>(in.size());
+    auto wcharcnt = MultiByteToWideChar(
+        CP_UTF8, MB_ERR_INVALID_CHARS, in.c_str(), isize, nullptr, 0);
+    if (wcharcnt <= 0) {
+#ifdef LOGERR
+        LOGERR("utf8towchar: conversion error for [" << in << "]\n");
+#endif
+        return false;
+    }
+    if (wcharcnt + 1 >  int(wcharsavail)) {
+#ifdef LOGERR
+        LOGERR("utf8towchar: not enough space\n");
+#endif
+        return false;
+    }
+    wcharcnt = MultiByteToWideChar(
+        CP_UTF8, MB_ERR_INVALID_CHARS, in.c_str(), isize, out, wcharsavail);
+    if (wcharcnt <= 0) {
+#ifdef LOGERR
+        LOGERR("utf8towchar: conversion error for [" << in << "]\n");
+#endif
+        return false;
+    }
+    out[wcharcnt] = 0;
+    return true;
+}
+
+std::unique_ptr<wchar_t[]> utf8towchar(const std::string& in)
+{
+    int isize = static_cast<int>(in.size());
+    // Note that as we supply in.size(), mbtowch computes the size
+    // without a terminating 0 (and won't write in the second call of
+    // course). We take this into account by allocating one more and
+    // terminating the output.
+    auto wcharcnt = MultiByteToWideChar(
+        CP_UTF8, MB_ERR_INVALID_CHARS, in.c_str(), isize, nullptr, 0);
+    if (wcharcnt <= 0) {
+#ifdef LOGERR
+        LOGERR("utf8towchar: conversion error for [" << in << "]\n");
+#endif
+        return std::unique_ptr<wchar_t[]>();
+    }
+    auto buf = std::make_unique<wchar_t[]>(wcharcnt+1);
+
+    wcharcnt = MultiByteToWideChar(
+        CP_UTF8, MB_ERR_INVALID_CHARS, in.c_str(), isize, buf.get(), wcharcnt);
+    if (wcharcnt <= 0) {
+#ifdef LOGERR
+        LOGERR("utf8towchar: conversion error for [" << in << "]\n");
+#endif
+        return std::unique_ptr<wchar_t[]>();
+    }
+    buf[wcharcnt] = 0;
+    return buf;
+}
+#endif // _WIN32
+
 int stringicmp(const std::string& s1, const std::string& s2)
 {
     return strcasecmp(s1.c_str(), s2.c_str());
@@ -68,7 +198,7 @@ std::string stringtolower(const std::string& i)
 
 void stringtoupper(std::string& io)
 {
-    std::transform(io.begin(), io.end(), io.begin(), [](unsigned char c) { return std::toupper(c); });
+    std::transform(io.begin(), io.end(), io.begin(), [](unsigned char c) {return std::toupper(c);});
 }
 
 std::string stringtoupper(const std::string& i)
@@ -140,9 +270,16 @@ int stringuppercmp(const std::string& s1, const std::string& s2)
     return size1 == size2 ? 0 : 1;
 }
 
-bool beginswith(const std::string& big, const std::string& small)
+bool beginswith(const std::string& bg, const std::string& sml)
 {
-    return big.compare(0, small.size(), small) == 0;
+    return bg.compare(0, sml.size(), sml) == 0;
+}
+bool endswith(const std::string& bg, const std::string& sml)
+{
+    if (bg.size() >= sml.size()) {
+        return (!bg.compare (bg.length() - sml.length(), sml.length(), sml));
+    }
+    return false;
 }
 
 template <class T> bool stringToStrings(const std::string& s, T& tokens, const std::string& addseps)
@@ -279,7 +416,7 @@ template <class T> void stringsToString(const T& tokens, std::string& s)
         }
         s.append(1, ' ');
     }
-    s.resize(s.size()-1);
+    s.pop_back();
 }
 
 template <class T> std::string stringsToString(const T& tokens)
@@ -324,7 +461,7 @@ template <class T> std::string commonprefix(const T& values)
     if (values.size() == 1)
         return *values.begin();
     unsigned int i = 0;
-    for (;;i++) {
+    for (;;++i) {
         auto it = values.begin();
         if (it->size() <= i) {
             goto out;
@@ -597,8 +734,7 @@ std::string makeCString(const std::string& in)
 // Substitute printf-like percent cmds inside a string
 bool pcSubst(const std::string& in, std::string& out, const std::map<char, std::string>& subs)
 {
-    std::string::const_iterator it;
-    for (it = in.begin(); it != in.end(); it++) {
+    for (auto it = in.begin(); it != in.end(); ++it) {
         if (*it == '%') {
             if (++it == in.end()) {
                 out += '%';
@@ -625,8 +761,8 @@ bool pcSubst(const std::string& in, std::string& out,
              const std::function<std::string(const std::string&)>& mapper)
 {
     out.erase();
-    std::string::size_type i;
-    for (i = 0; i < in.size(); i++) {
+
+    for (std::string::size_type i = 0; i < in.size(); ++i) {
         if (in[i] == '%') {
             if (++i == in.size()) {
                 out += '%';
@@ -642,7 +778,7 @@ bool pcSubst(const std::string& in, std::string& out,
                     out += std::string("%(");
                     break;
                 }
-                std::string::size_type j = in.find_first_of(')', i);
+                auto j = in.find_first_of(')', i);
                 if (j == std::string::npos) {
                     // ??concatenate remaining part and stop
                     out += in.substr(i - 2);
@@ -780,7 +916,7 @@ std::string breakIntoLines(const std::string& in, unsigned int ll, unsigned int
                     ss = query;
                 }
             } else {
-                ss = ss.substr(0, pos + 1);
+                ss.resize(pos + 1);
             }
         }
         // This cant happen, but anyway. Be very sure to avoid an infinite loop
@@ -798,95 +934,6 @@ std::string breakIntoLines(const std::string& in, unsigned int ll, unsigned int
     return oq;
 }
 
-// Date is Y[-M[-D]]
-static bool parsedate(std::vector<std::string>::const_iterator& it,
-                      std::vector<std::string>::const_iterator end, DateInterval *dip)
-{
-    dip->y1 = dip->m1 = dip->d1 = dip->y2 = dip->m2 = dip->d2 = 0;
-    if (it->length() > 4 || it->empty() ||
-        it->find_first_not_of("0123456789") != std::string::npos) {
-        return false;
-    }
-    if (it == end || sscanf(it++->c_str(), "%d", &dip->y1) != 1) {
-        return false;
-    }
-    if (it == end || *it == "/") {
-        return true;
-    }
-    if (*it++ != "-") {
-        return false;
-    }
-
-    if (it->length() > 2 || it->empty() ||
-        it->find_first_not_of("0123456789") != std::string::npos) {
-        return false;
-    }
-    if (it == end || sscanf(it++->c_str(), "%d", &dip->m1) != 1) {
-        return false;
-    }
-    if (it == end || *it == "/") {
-        return true;
-    }
-    if (*it++ != "-") {
-        return false;
-    }
-
-    if (it->length() > 2 || it->empty() ||
-        it->find_first_not_of("0123456789") != std::string::npos) {
-        return false;
-    }
-    if (it == end || sscanf(it++->c_str(), "%d", &dip->d1) != 1) {
-        return false;
-    }
-
-    return true;
-}
-
-// Called with the 'P' already processed. Period ends at end of string
-// or at '/'. We dont' do a lot effort at validation and will happily
-// accept 10Y1Y4Y (the last wins)
-static bool parseperiod(std::vector<std::string>::const_iterator& it,
-                        std::vector<std::string>::const_iterator end, DateInterval *dip)
-{
-    dip->y1 = dip->m1 = dip->d1 = dip->y2 = dip->m2 = dip->d2 = 0;
-    while (it != end) {
-        int value;
-        if (it->find_first_not_of("0123456789") != std::string::npos) {
-            return false;
-        }
-        if (sscanf(it++->c_str(), "%d", &value) != 1) {
-            return false;
-        }
-        if (it == end || it->empty()) {
-            return false;
-        }
-        switch (it->at(0)) {
-        case 'Y':
-        case 'y':
-            dip->y1 = value;
-            break;
-        case 'M':
-        case 'm':
-            dip->m1 = value;
-            break;
-        case 'D':
-        case 'd':
-            dip->d1 = value;
-            break;
-        default:
-            return false;
-        }
-        it++;
-        if (it == end) {
-            return true;
-        }
-        if (*it == "/") {
-            return true;
-        }
-    }
-    return true;
-}
-
 #ifdef _WIN32
 static int setenv(const char* name, const char* value, int overwrite)
 {
@@ -902,12 +949,16 @@ static void unsetenv(const char* name)
 {
     _putenv_s(name, "");
 }
-#endif
-
+#ifdef _MSC_VER
+time_t portable_timegm(struct tm *tm)
+{
+    return _mkgmtime(tm);
+}
+#else
 time_t portable_timegm(struct tm *tm)
 {
     time_t ret;
-    char *tz;
+    const char *tz;
 
     tz = getenv("TZ");
     setenv("TZ", "", 1);
@@ -921,201 +972,14 @@ time_t portable_timegm(struct tm *tm)
     tzset();
     return ret;
 }
-
-#if 0
-static void cerrdip(const std::string& s, DateInterval *dip)
+#endif // End Windows not msvc
+#else // -> !Windows
+time_t portable_timegm(struct tm *tm)
 {
-    cerr << s << dip->y1 << "-" << dip->m1 << "-" << dip->d1 << "/"
-         << dip->y2 << "-" << dip->m2 << "-" << dip->d2
-         << endl;
+    return timegm(tm);
 }
 #endif
 
-// Compute date + period. Won't work out of the unix era.
-// or pre-1970 dates. Just convert everything to unixtime and
-// seconds (with average durations for months/years), add and convert
-// back
-static bool addperiod(DateInterval *dp, DateInterval *pp)
-{
-    // Create a struct tm with possibly non normalized fields and let
-    // timegm sort it out
-    struct tm tm = {};
-    tm.tm_year = dp->y1 - 1900 + pp->y1;
-    tm.tm_mon = dp->m1 + pp->m1 - 1;
-    tm.tm_mday = dp->d1 + pp->d1;
-    time_t tres = mktime(&tm);
-    localtime_r(&tres, &tm);
-    dp->y1 = tm.tm_year + 1900;
-    dp->m1 = tm.tm_mon + 1;
-    dp->d1 = tm.tm_mday;
-    //cerrdip("Addperiod return", dp);
-    return true;
-}
-int monthdays(int mon, int year)
-{
-    switch (mon) {
-        // We are returning a few too many 29 days februaries, no problem
-    case 2:
-        return (year % 4) == 0 ? 29 : 28;
-    case 1:
-    case 3:
-    case 5:
-    case 7:
-    case 8:
-    case 10:
-    case 12:
-        return 31;
-    default:
-        return 30;
-    }
-}
-bool parsedateinterval(const std::string& s, DateInterval *dip)
-{
-    std::vector<std::string> vs;
-    dip->y1 = dip->m1 = dip->d1 = dip->y2 = dip->m2 = dip->d2 = 0;
-    DateInterval p1, p2, d1, d2;
-    p1 = p2 = d1 = d2 = *dip;
-    bool hasp1 = false, hasp2 = false, hasd1 = false, hasd2 = false,
-        hasslash = false;
-
-    if (!stringToStrings(s, vs, "PYMDpymd-/")) {
-        return false;
-    }
-    if (vs.empty()) {
-        return false;
-    }
-
-    auto it = vs.cbegin();
-    if (*it == "P" || *it == "p") {
-        it++;
-        if (!parseperiod(it, vs.end(), &p1)) {
-            return false;
-        }
-        hasp1 = true;
-        //cerrdip("p1", &p1);
-        p1.y1 = -p1.y1;
-        p1.m1 = -p1.m1;
-        p1.d1 = -p1.d1;
-    } else if (*it == "/") {
-        hasslash = true;
-        goto secondelt;
-    } else {
-        if (!parsedate(it, vs.end(), &d1)) {
-            return false;
-        }
-        hasd1 = true;
-    }
-
-    // Got one element and/or /
-secondelt:
-    if (it != vs.end()) {
-        if (*it != "/") {
-            return false;
-        }
-        hasslash = true;
-        it++;
-        if (it == vs.end()) {
-            // ok
-        } else if (*it == "P" || *it == "p") {
-            it++;
-            if (!parseperiod(it, vs.end(), &p2)) {
-                return false;
-            }
-            hasp2 = true;
-        } else {
-            if (!parsedate(it, vs.end(), &d2)) {
-                return false;
-            }
-            hasd2 = true;
-        }
-    }
-
-    // 2 periods dont' make sense
-    if (hasp1 && hasp2) {
-        return false;
-    }
-    // Nothing at all doesn't either
-    if (!hasp1 && !hasd1 && !hasp2 && !hasd2) {
-        return false;
-    }
-
-    // Empty part means today IF other part is period, else means
-    // forever (stays at 0)
-    time_t now = time(nullptr);
-    struct tm *tmnow = gmtime(&now);
-    if ((!hasp1 && !hasd1) && hasp2) {
-        d1.y1 = 1900 + tmnow->tm_year;
-        d1.m1 = tmnow->tm_mon + 1;
-        d1.d1 = tmnow->tm_mday;
-        hasd1 = true;
-    } else if ((!hasp2 && !hasd2) && hasp1) {
-        d2.y1 = 1900 + tmnow->tm_year;
-        d2.m1 = tmnow->tm_mon + 1;
-        d2.d1 = tmnow->tm_mday;
-        hasd2 = true;
-    }
-
-    // Incomplete dates have different meanings depending if there is
-    // a period or not (actual or infinite indicated by a / + empty)
-    //
-    // If there is no explicit period, an incomplete date indicates a
-    // period of the size of the uncompleted elements. Ex: 1999
-    // actually means 1999/P12M
-    //
-    // If there is a period, the incomplete date should be extended
-    // to the beginning or end of the unspecified portion. Ex: 1999/
-    // means 1999-01-01/ and /1999 means /1999-12-31
-    if (hasd1) {
-        if (!(hasslash || hasp2)) {
-            if (d1.m1 == 0) {
-                p2.m1 = 12;
-                d1.m1 = 1;
-                d1.d1 = 1;
-            } else if (d1.d1 == 0) {
-                d1.d1 = 1;
-                p2.d1 = monthdays(d1.m1, d1.y1);
-            }
-            hasp2 = true;
-        } else {
-            if (d1.m1 == 0) {
-                d1.m1 = 1;
-                d1.d1 = 1;
-            } else if (d1.d1 == 0) {
-                d1.d1 = 1;
-            }
-        }
-    }
-    // if hasd2 is true we had a /
-    if (hasd2) {
-        if (d2.m1 == 0) {
-            d2.m1 = 12;
-            d2.d1 = 31;
-        } else if (d2.d1 == 0) {
-            d2.d1 = monthdays(d2.m1, d2.y1);
-        }
-    }
-    if (hasp1) {
-        // Compute d1
-        d1 = d2;
-        if (!addperiod(&d1, &p1)) {
-            return false;
-        }
-    } else if (hasp2) {
-        // Compute d2
-        d2 = d1;
-        if (!addperiod(&d2, &p2)) {
-            return false;
-        }
-    }
-
-    dip->y1 = d1.y1;
-    dip->m1 = d1.m1;
-    dip->d1 = d1.d1;
-    dip->y2 = d2.y1;
-    dip->m2 = d2.m1;
-    dip->d2 = d2.d1;
-    return true;
-}
 
 std::string hexprint(const std::string& in, char separ)
 {
@@ -1123,7 +987,7 @@ std::string hexprint(const std::string& in, char separ)
     out.reserve(separ ? (3 *in.size()) : (2 * in.size()));
     static const char hex[]="0123456789abcdef";
     auto cp = reinterpret_cast<const unsigned char*>(in.c_str());
-    for (unsigned int i = 0; i < in.size(); i++) {
+    for (unsigned int i = 0; i < in.size(); ++i) {
         out.append(1, hex[cp[i] >> 4]);
         out.append(1, hex[cp[i] & 0x0f]);
         if (separ && i != in.size() - 1)
@@ -1243,22 +1107,18 @@ public:
 };
 
 // Substitute one instance of regular expression
-std::string SimpleRegexp::simpleSub(
-    const std::string& in, const std::string& repl)
+std::string SimpleRegexp::simpleSub(const std::string& in, const std::string& repl)
 {
     if (!ok()) {
         return {};
     }
-
-    int err;
-    if ((err = regexec(&m->expr, in.c_str(),
-                       m->nmatch + 1, &m->matches[0], 0))) {
+    if (int err = regexec(&m->expr, in.c_str(), m->nmatch + 1, &m->matches[0], 0)) {
+        PRETEND_USE(err);
 #if SIMPLESUB_DBG
         const int ERRSIZE = 200;
         char errbuf[ERRSIZE + 1];
         regerror(err, &expr, errbuf, ERRSIZE);
-        std::cerr << "simpleSub: regexec(" << sexp << ") failed: "
-                  <<  errbuf << "\n";
+        std::cerr << "simpleSub: regexec(" << sexp << ") failed: " <<  errbuf << "\n";
 #endif
         return in;
     }
@@ -1346,12 +1206,11 @@ std::string valToString(const std::vector<CharFlags>& flags, unsigned int val)
 static inline int h2d(int c) {
     if ('0' <= c && c <= '9')
         return c - '0';
-    else if ('A' <= c && c <= 'F')
+    if ('A' <= c && c <= 'F')
         return 10 + c - 'A';
-    else if ('a' <= c && c <= 'f')
+    if ('a' <= c && c <= 'f')
         return 10 + c - 'a';
-    else 
-        return -1;
+    return -1;
 }
 
 std::string pc_decode(const std::string &in)
@@ -1362,7 +1221,7 @@ std::string pc_decode(const std::string &in)
     out.reserve(in.size());
     const char *cp = in.c_str();
     std::string::size_type i = 0;
-    for (; i < in.size() - 2; i++) {
+    for (; i < in.size() - 2; ++i) {
         if (cp[i] == '%') {
             int d1 = h2d(cp[i+1]);
             int d2 = h2d(cp[i+2]);
@@ -1419,6 +1278,18 @@ bool parseHTTPRanges(const std::string& ranges, std::vector<std::pair<int64_t, i
     return true;
 }
 
+void millisleep(int millis)
+{
+#ifdef _WIN32
+    Sleep(millis);
+#else
+    struct timespec spec;
+    spec.tv_sec = millis / 1000;
+    spec.tv_nsec = (millis % 1000) * 1000000;
+    nanosleep(&spec, nullptr);
+#endif
+}
+
 // Initialization for static stuff to be called from main thread before going
 // multiple
 void smallut_init_mt()
